
indexing
   author: "Miron Brezuleanu"
   purpose: "wrap common behaviour for object that store/retrieve themselves;"
	    
deferred class EXG_STORABLE
   
feature -- common behaviour
   
   load is
	 -- will use a file selector to load a file
      local
	 efsm: EXG_FILE_SELECTION_MODAL
	 emb: EXG_MESSAGE_BOX
	 file: STD_FILE_READ
      do
	 !!efsm.make
	 efsm.set_title("Open ...")
	 if open_path /= Void then
	    efsm.set_filename (open_path)
	 end	 
	 efsm.run
	 if equal(efsm.answer, "ok") then
	    !!file.connect_to(efsm.filename)
	    if not file.is_connected then
	       !!emb.make
	       emb.set_message("Can't open " 
			       + efsm.filename + " for reading !")
	       emb.run
	    else
	       file.read_line
	       if equal(file.last_string, signature) then
		  load_from_file(file)
		  open_split(efsm.filename)
		  debug
		     print(filename + "%N")
		  end
	       else
		  !!emb.make
		  emb.set_message("File format not recognized!")
		  emb.run
	       end
	       file.disconnect
	    end
	 end
      end
	  
   direct_load(file_location: STRING) is
	 -- will directly load a called file
      local
	 emb: EXG_MESSAGE_BOX
	 file: STD_FILE_READ
      do
	    !!file.connect_to(file_location)
	    if not file.is_connected then
	       !!emb.make
	       emb.set_message("Can't open " 
			       + file_location + " for reading !")
	       emb.run
	    else
	       file.read_line
	       if equal(file.last_string, signature) then
		  load_from_file(file)
		  open_split(file_location)
		  debug
		     print(filename + "%N")
		  end
	       else
		  !!emb.make
		  emb.set_message("File format not recognized!")
		  emb.run
	       end
	       file.disconnect
	    end
      end
   
   save is
	 -- like `load', but does the reversed action
      local
	 efsm: EXG_FILE_SELECTION_MODAL
	 file: STD_FILE_WRITE
	 emb: EXG_MESSAGE_BOX
      do
	 !!efsm.make
	 efsm.set_title("Save ...")
	 if save_path /= Void then
	    efsm.set_filename (save_path)
	 end	 
	 efsm.run
	 if equal(efsm.answer, "ok") then
	    !!file.connect_to(efsm.filename)
	    if not file.is_connected then
	       !!emb.make
	       emb.set_message("Can't open " 
			       + efsm.filename + " for writing !")
	       emb.run
	    else
	       file.put_string(signature + "%N")
	       save_to_file(file)
		   save_split(efsm.filename)
	       debug
		  print(filename + "%N")
	       end
	       file.disconnect
	    end
	 end
      end
   
feature -- the holes... (uncommon behaviour)
   
   load_from_file(file: STD_FILE_READ) is
	 -- will load data from the given file
      deferred
      end
	 
   save_to_file(file: STD_FILE_WRITE) is
	 -- like `load_from_file' but does the saving
      deferred
      end
   
   signature: STRING is
	 -- the string that should identify the file format; should 
	 -- not contain any newlines
      deferred
      end
feature -- name access

   filename: STRING
	 -- the last filename used in saving or loading

   save_path: STRING
     -- the path to the last saved file

   open_path: STRING
     -- the path to the last opened file

   save_split(full_path: STRING) is
     -- set the filename and path to the saved
	 -- file during a save
     local
	   idx: INTEGER
	   file: STRING
	   split_at: CHARACTER
   	 do
	   file := "temp"
	   file.copy(full_path)
	   if file.has('\') then
	     split_at := '\'
	   else
	     split_at := '/'
	   end
	   file.reverse
	   idx := file.index_of(split_at)
	   file.head(idx - 1)
	   file.reverse
	   filename := file 
	   full_path.remove_suffix(filename)
	   save_path := full_path
	 end

   open_split(full_path: STRING) is
     -- set the filename and path to the opened
	 -- file during an open
     local
	   idx: INTEGER
	   file: STRING
	   split_at: CHARACTER
   	 do
	   file := "temp"
	   file.copy(full_path)
	   if file.has('\') then
	     split_at := '\'
	   else
	     split_at := '/'
	   end
	   file.reverse
	   idx := file.index_of(split_at)
	   file.head(idx - 1)
	   file.reverse
	   filename := file
	   full_path.remove_suffix(filename)
	   open_path := full_path
	 end
	 
end
